@@ -8,9 +8,9 @@ use Log::ger;
 use Perinci::Exporter;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2022-05-06'; # DATE
+our $DATE = '2022-05-08'; # DATE
 our $DIST = 'App-ImageMagickUtils'; # DIST
-our $VERSION = '0.015'; # VERSION
+our $VERSION = '0.016'; # VERSION
 
 our %SPEC;
 
@@ -84,9 +84,10 @@ or (if downsizing is done):
 _
     args => {
         %argspec0_files,
-        q => {
+        quality => {
             schema => ['int*', between=>[0,100]],
             default => 40,
+            cmdline_aliases => {q=>{}},
         },
         downsize_to => {
             schema => ['str*', in=>['', '640', '800', '1024', '1536', '2048']],
@@ -107,6 +108,34 @@ _
                 2048          => {summary=>"Shortcut for --downsize-to=2048", is_flag=>1, code=>sub {$_[0]{downsize_to} = '2048'}},
             },
         },
+        skip_whatsapp => {
+            summary => 'Skip WhatsApp images',
+            'summary.alt.bool.not' => 'Do not skip WhatsApp images',
+            schema => 'bool*',
+            default => 1,
+            description => <<'_',
+
+By default, assuming that WhatsApp already compresses images, when given a
+filename that matches a WhatsApp image filename, e.g. `IMG-20220508-WA0001.jpg`
+(will be checked using <pm:Regexp::Pattern::Filename::Image::WhatsApp>), will
+skip downsizing. The `--no-skip-whatsapp` option will process such filenames
+nevertheless.
+
+_
+        },
+        skip_downsized => {
+            summary => 'Skip previously downsized images',
+            'summary.alt.bool.not' => 'Do not skip previously downsized images',
+            schema => 'bool*',
+            default => 1,
+            description => <<'_',
+
+By default, when given a filename that looks like it's already downsized, e.g.
+`foo.1024-q40.jpg` or `foo.q40.jpg`, will skip downsizing. The
+`--no-skip-downsized` option will process such filenames nevertheless.
+
+_
+        },
         %argspecs_delete,
     },
     args_rels => \%args_rels,
@@ -140,6 +169,8 @@ sub downsize_image {
 
     my $convert_path = File::Which::which("convert");
     my $downsize_to = $args{downsize_to};
+    my $skip_whatsapp = $args{skip_whatsapp} // 1;
+    my $skip_downsized = $args{skip_downsized} // 1;
 
     unless ($args{-dry_run}) {
         return [400, "Cannot find convert in path"] unless defined $convert_path;
@@ -148,23 +179,39 @@ sub downsize_image {
 
     my ($num_files, $num_success) = (0, 0);
     my $trash;
+  FILE:
     for my $file (@{$args{files}}) {
         log_info "Processing file %s ...", $file;
         $num_files++;
 
         unless (-f $file) {
             log_error "No such file %s, skipped", $file;
-            next;
+            next FILE;
         }
 
         #my $res = Filename::Image::check_image_filename(filename => $file);
         my ($width, $height, $fmt) = Image::Size::imgsize($file);
         unless ($width) {
             log_error "Filename '%s' is not image (%s), skipped", $file, $fmt;
-            next;
+            next FILE;
         }
 
-        my $q = $args{q} // 40;
+        if ($skip_whatsapp) {
+            require Regexp::Pattern::Filename::Image::WhatsApp;
+            if ($file =~ $Regexp::Pattern::Filename::Image::WhatsApp::RE{filename_image_whatsapp}{pat}) {
+                log_info "Filename '%s' looks like a WhatsApp image, skip downsizing due to --skip-whatsapp option is in effect", $file;
+                next FILE;
+            }
+        }
+
+        if ($skip_downsized) {
+            if ($file =~ /\.(?:\d+-)?q(?:\d{1,3})\.\w+\z/) {
+                log_info "Filename '%s' looks like it's already downsized, skip downsizing due to --skip-downsized option is in effect", $file;
+                next FILE;
+            }
+        }
+
+        my $q = $args{quality} // 40;
         my @convert_args = (
             $file,
         );
@@ -192,7 +239,8 @@ sub downsize_image {
 
         if ($args{-dry_run}) {
             log_info "[DRY-RUN] Running $convert_path with args %s ...", \@convert_args;
-            next;
+            $num_success++;
+            next FILE;
         }
 
         IPC::System::Options::system(
@@ -344,7 +392,7 @@ App::ImageMagickUtils - Utilities related to ImageMagick
 
 =head1 VERSION
 
-This document describes version 0.015 of App::ImageMagickUtils (from Perl distribution App-ImageMagickUtils), released on 2022-05-06.
+This document describes version 0.016 of App::ImageMagickUtils (from Perl distribution App-ImageMagickUtils), released on 2022-05-08.
 
 =head1 DESCRIPTION
 
@@ -532,7 +580,25 @@ C<--dont-downsize> on the CLI.
 
 =item * B<files>* => I<array[filename]>
 
-=item * B<q> => I<int> (default: 40)
+=item * B<quality> => I<int> (default: 40)
+
+=item * B<skip_downsized> => I<bool> (default: 1)
+
+Skip previously downsized images.
+
+By default, when given a filename that looks like it's already downsized, e.g.
+C<foo.1024-q40.jpg> or C<foo.q40.jpg>, will skip downsizing. The
+C<--no-skip-downsized> option will process such filenames nevertheless.
+
+=item * B<skip_whatsapp> => I<bool> (default: 1)
+
+Skip WhatsApp images.
+
+By default, assuming that WhatsApp already compresses images, when given a
+filename that matches a WhatsApp image filename, e.g. C<IMG-20220508-WA0001.jpg>
+(will be checked using L<Regexp::Pattern::Filename::Image::WhatsApp>), will
+skip downsizing. The C<--no-skip-whatsapp> option will process such filenames
+nevertheless.
 
 =item * B<trash_original> => I<bool>
 
